#! /usr/bin/python
# -*- coding=utf-8 -*-

"""
    This file is part of Gnome 3 background slideshow.

    Gnome 3 background slideshow is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Gnome 3 background slideshow is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gnome 3 background slideshow.  If not, see <http://www.gnu.org/licenses/>.
"""

from gi.repository import Gtk, Gdk
import DefaultSettings, FileDialogs, os

class DConfWidget(object):
   CHANGED_SIGNALS = []
   def __init__(self, schema, key):
      self._schema = schema
      self._key = key
      self.set_value(schema[key])
   def start_watching(self):
      for i in self.CHANGED_SIGNALS:
         self.connect(i, self._on_widget_value_changed)
      self._schema.connect("changed::"+self._key, self._on_dconf_value_changed)
   def _on_dconf_value_changed(self, schema, key):
      self.set_value(schema[key])
   def _on_widget_value_changed(self, *args):
      value = self.get_value()
      if value!=None:
         getattr(self._schema, "set_"+self.VALUE_TYPE)(self._key, value)
      
class DConfComboxBox(Gtk.ComboBox, DConfWidget):
   CHANGED_SIGNALS = ['changed']
   VALUE_TYPE = "string"
   def __init__(self, schema, key):
      Gtk.ComboBox.__init__(self)
      self._model = Gtk.ListStore(str, str)
      self.set_model(self._model)
      for i in DefaultSettings.LIST_VALUES[key]:
         self._model.append(i)
      r = Gtk.CellRendererText()
      self.pack_start(r, False)
      self.add_attribute(r,"text",1)
      DConfWidget.__init__(self, schema, key)
   def set_value(self, value):
      for i in range(len(self._model)):
         if self._model[i][0]==value:
            self.set_active(i)
            return
   def get_value(self):
      return self._model[self.get_active()][0]

class DConfFilenameChooser(Gtk.FileChooserButton, DConfWidget):
   CHANGED_SIGNALS = ["file-set"]
   VALUE_TYPE = "string"
   def __init__(self, schema, key, dialog=None):
      Gtk.FileChooserButton.__init__(self, dialog=dialog)
      DConfWidget.__init__(self, schema, key)
   def set_value(self, value):
      self.set_filename(value[7:])
   def get_value(self):
      return "file://"+self.get_filename()

class DConfImageFileChooser(DConfFilenameChooser):
   def __init__(self, schema, key, parent, pixbuf_cache):
      DConfFilenameChooser.__init__(self, schema, key, FileDialogs.ImageFileChooserDialog(parent, pixbuf_cache))

class FolderChooserDialog(Gtk.FileChooserDialog):
   def __init__(self):
      Gtk.FileChooserDialog.__init__(self, action=Gtk.FileChooserAction.SELECT_FOLDER)
      self.add_button(Gtk.STOCK_OPEN, Gtk.ResponseType.OK)
      self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
   def run(self):
      self.show_all()
      resp = Gtk.FileChooserDialog.run(self)
      if resp == Gtk.ResponseType.OK:
         res = self.get_filename()
      else:
         res = None
      self.hide()
      return res

class DConfFolderChooser(Gtk.Button, DConfWidget):
   CHANGED_SIGNALS = ["folder_changed"]
   VALUE_TYPE = "string"
   def __init__(self, schema, key):
      Gtk.Button.__init__(self)
      self._folder = None
      self._signals = {}
      self._file_chooser_dialog = FolderChooserDialog()
      hbox = Gtk.HBox()
      self.add(hbox)
      img = Gtk.Image()
      img.set_from_icon_name("folder", Gtk.IconSize.BUTTON)
      hbox.pack_start(img, False, False, 0)
      self._label = Gtk.Label()
      self._label.set_alignment(0, 0.5)
      hbox.pack_start(self._label, True, True, 5)
      DConfWidget.__init__(self, schema, key)
      Gtk.Button.connect(self, "clicked", self._on_click)
   def _on_click(self, widget):
      res = self._file_chooser_dialog.run()
      if res != None:
         self.set_value("file://" + res)
         self._emit("folder_changed", res)
   def set_value(self, value):
      self._folder = value
      self._update_label()
      self._file_chooser_dialog.set_current_folder(value[6:])
      self._file_chooser_dialog.set_filename(value[6:])
   def get_value(self):
      return self._folder
   def _update_label(self):
      value = self.get_value()
      if value==None:
         label = _("(None)")
      else:
         label = os.path.split(value)[-1]
      self._label.set_text(label)
   def connect(self, signal_name, handler, *params):
      self._signals[signal_name] = {'handler': handler, 'params': params}
   def _emit(self, signal_name, *params):
      if signal_name in self._signals.keys():
         handler = self._signals[signal_name]['handler']
         handler(params + self._signals[signal_name]['params'])

class DConfColorChooser(Gtk.ColorButton, DConfWidget):
   CHANGED_SIGNALS = ["color-set"]
   VALUE_TYPE = "string"
   def __init__(self, schema, key):
      Gtk.ColorButton.__init__(self)
      DConfWidget.__init__(self, schema, key)
   def get_value(self):
      return self.get_color().to_string()
   def set_value(self, value):
      self.set_color(Gdk.color_parse(value))

class DConfRadioButton(Gtk.RadioButton, DConfWidget):
   CHANGED_SIGNALS = ["toggled"]
   VALUE_TYPE = "string"
   def __init__(self, schema, key, value):
      Gtk.RadioButton.__init__(self)
      self._value = value
      DConfWidget.__init__(self, schema, key)
   def set_value(self, value):
      self.set_active(value==self._value)
   def get_value(self):
      if self.get_active():
         return self._value

class DConfToggleButton(Gtk.CheckButton, DConfWidget):
   CHANGED_SIGNALS = ["toggled"]
   VALUE_TYPE = "boolean"
   def __init__(self, schema, key):
      Gtk.CheckButton.__init__(self)
      DConfWidget.__init__(self, schema, key)
   def set_value(self, value):
      self.set_active(value)
   def get_value(self):
      return self.get_active()

class DConfIntegerWidget(Gtk.SpinButton, DConfWidget):
   CHANGED_SIGNALS = ["value-changed"]
   VALUE_TYPE = "int"
   def __init__(self, schema, key, min_value, max_value, small_inc=1, big_inc=10):
      Gtk.SpinButton.__init__(self)
      self.set_range(min_value, max_value)
      self.set_increments(small_inc, big_inc)
      DConfWidget.__init__(self, schema, key)

class DConfEntry(Gtk.Entry, DConfWidget):
   CHANGED_SIGNALS = ["changed"]
   VALUE_TYPE = "string"
   def __init__(self, schema, key):
      Gtk.Entry.__init__(self)
      DConfWidget.__init__(self, schema, key)
   def get_value(self):
      return self.get_text()
   def set_value(self, value):
      self.set_text(value)
