#! /usr/bin/python
# -*- coding=utf-8 -*-

"""
    This file is part of Gnome 3 background slideshow.

    Gnome 3 background slideshow is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Gnome 3 background slideshow is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gnome 3 background slideshow.  If not, see <http://www.gnu.org/licenses/>.
"""

from TrayIcon import TrayIcon
from gi.repository import Gtk, Gio, GObject
from ConfigDialog import ConfigDialog
from AboutDialog import AboutDialog
import random, os, commands, thread
from ThreadedVariable import ThreadedVariable
import clioptions
from PicturesListEditor.Editor import PicturesListEditor
from informations import *
from PixbufCache import PixbufCache
try:
   import mateconf
   MATECONF_SUPPORT = True
except:
   MATECONF_SUPPORT = False

class Applet(object):
   def __init__(self):
      self._gnome_background_schema = Gio.Settings("org.gnome.desktop.background")
      self._app_schema = Gio.Settings("apps.gnome3-background-slideshow")
      if MATECONF_SUPPORT:
         self._mateconf_client = mateconf.client_get_default()
      else:
         self._mateconf_client = None
      self._pixbuf_cache = PixbufCache()
      self._config_dialog = ConfigDialog(self, self._gnome_background_schema, self._app_schema, self._pixbuf_cache)
      if not clioptions.configure:
         self._tray_icon = TrayIcon(self)
         self._about_dialog=AboutDialog()
         self._pictures_list_editor = PicturesListEditor(self, self._app_schema, self._pixbuf_cache)
         self._timer = None
         self._pictures_list = []
         self.pictures_list_loaded = ThreadedVariable(True)
         self.restart_pictures_loading = ThreadedVariable(False)
         self.pictures_list_loading_progress = ThreadedVariable(0)
         self._file_types = {}
         for key in ["delay", "mode", "folder", "list-filename", "recursive", "show-tray-icon"]:
            self._app_schema.connect('changed::'+key, self._on_app_config_changed)
         if MATECONF_SUPPORT:
            for key in ["picture-uri", "picture-options", "color-shading-type", "primary-color", "secondary-color"]:
               self._gnome_background_schema.connect('changed::'+key, self._on_gnome_config_changed)
            
            # Not handled yet due to bug in mateconf library
            #self._mateconf_client.add_dir("/desktop/mate/background", mateconf.CLIENT_PRELOAD_NONE)
            #self._mateconf_client.notify_add("/desktop/mate/background/picture_filename", self._on_mateconf_changed)
   def _on_mateconf_changed(self, client, _, entry, args):
      # Not handled yet due to bug in mateconf library
      pass
   def _on_gnome_config_changed(self, schema, key):
      if key=="picture-uri":
         self._mateconf_client.set_string("/desktop/mate/background/picture_filename", self._gnome_background_schema["picture-uri"][7:])
      elif key in ["primary-color", "secondary-color"]:
         self._mateconf_client.set_string("/desktop/mate/background/"+key.replace("-", "_"), self._gnome_background_schema[key])
      elif key=="color-shading-type":
         if self._gnome_background_schema[key] in ["vertical", "horizontal"]:
            value = self._gnome_background_schema[key]+"-gradient"
         else:
            value = self._gnome_background_schema[key]
         self._mateconf_client.set_string("/desktop/mate/background/"+key.replace("-", "_"), value)
      elif key=="picture-options":
         value = self._gnome_background_schema[key]
         self._mateconf_client.set_bool("/desktop/mate/background/draw_background", value!="none")
         self._mateconf_client.set_string("/desktop/mate/background/picture_options", value)
   def _on_app_config_changed(self, schema, key):
      if key=='delay':
         self._init_timer()
      elif key=="mode":
         if schema['mode'] in ['list', 'folder']:
            self._reinit_list()
         elif self._timer:
            GObject.source_remove(self._timer)
      elif (key=="recursive" and self._app_schema['mode']=="folder") or (key=="folder" and self._app_schema['mode']=="folder") or (key=="list-filename" and self._app_schema['mode']=="list"):
         self._reinit_list()
      elif key=="show-tray-icon":
         self._tray_icon.set_visible(schema["show-tray-icon"])
   def _list_pictures(self):
      self.pictures_list_loading_progress.set(-1)
      pictures_list = []
      try:
         if self._app_schema['mode']=="list":
            f = open(self._app_schema["list-filename"][7:])
            pictures_list = f.read().splitlines()
            f.close()
         else:
            recursive = self._app_schema["recursive"]
            list_folders = [self._app_schema["folder"][7:]]
            while len(list_folders)>0:
               if self.restart_pictures_loading.get():
                  break
               folder_path = list_folders[0]
               del list_folders[0]
               for i in os.listdir(folder_path):
                  filename = os.path.join(folder_path, i)
                  if os.path.isfile(filename):
                     pictures_list.append(filename)
                  elif recursive and os.path.isdir(filename):
                     list_folders.append(filename)
      except:
         pass
      i = 0
      p = 0
      n = len(pictures_list)
      while i<len(pictures_list):
         if self.restart_pictures_loading.get():
            break
         if self.is_picture(pictures_list[i]):
            i += 1
         else:
            del pictures_list[i]
         p += 1
         self.pictures_list_loading_progress.set(1.*p/n)
      if self.restart_pictures_loading.get():
         self.restart_pictures_loading.set(False)
         self._list_pictures()
      else:
         self._pictures_list = pictures_list
         self.pictures_list_loading_progress.set(1)
         self.pictures_list_loaded.set(True)
   def is_picture(self, filename):
      try:
         mtime = os.path.getmtime(filename)
         load_type = True
         if filename in self._file_types and self._file_types[filename]["mtime"]==mtime:
            load_type = False
         if load_type:
            self._file_types[filename] = {"mtime": mtime, "is_picture": commands.getoutput("file -bi \"%s\""%filename).startswith("image/")}
         return self._file_types[filename]["is_picture"]
      except:
         return False
   def _init_timer(self):
      if self._timer:
         GObject.source_remove(self._timer)
      self._timer = GObject.timeout_add_seconds(60*self._app_schema['delay'], self.next_picture)
   def next_picture(self):
      if self._app_schema['mode']=="picture":
         return
      try:
         self._gnome_background_schema.set_string('picture-uri', 'file://'+self._pictures_list[random.randrange(len(self._pictures_list))])
      finally:
         return True
   def _enable_list_commands(self):
      if not clioptions.configure:
         self._tray_icon.enable_list_options()
   def _disable_list_commands(self):
      if not clioptions.configure:
         self._tray_icon.disable_list_options()
   def _on_pictures_list_loaded(self):
      self._init_timer()
      self.next_picture()
      self._enable_list_commands()
   def _reinit_list(self):
      self._disable_list_commands()
      if self.pictures_list_loaded.get():
         self.pictures_list_loaded.set(False)
         thread.start_new_thread(self._list_pictures, ())
         GObject.timeout_add(100, self._check_pictures_list_loaded)
      else:
         self.restart_pictures_loading.set(True)
   def _check_pictures_list_loaded(self):
      if self.pictures_list_loaded.get():
         self._on_pictures_list_loaded()
         return False
      else:
         return True
   def run(self):
      if clioptions.configure:
         self.show_properties()
      else:
         GObject.threads_init()
         if self._app_schema['mode'] in ['list', 'folder']:
            self._reinit_list()
         self._tray_icon.set_visible(self._app_schema["show-tray-icon"])
         Gtk.main()
   def quit(self):
      Gtk.main_quit()
   def show_about(self):
      self._about_dialog.run()
   def show_properties(self):
      self._config_dialog.run()
   def rec_mkdir(self, path):
      if os.path.exists(path):
         return
      self.rec_mkdir(os.path.split(path)[0])
      os.mkdir(path)
   def edit_list(self):
      filename = None
      if self._app_schema["mode"]=="list":
         filename = self._app_schema["list-filename"][7:]
         if not os.path.exists(filename):
            filename = None
      if filename==None:
         path = os.path.join(os.getenv("HOME"), "."+UNIX_APP_NAME, "lists")
         self.rec_mkdir(path)
         i = 1
         filename = os.path.join(path, str(i))
         while os.path.exists(filename):
            i += 1
            filename = os.path.join(path, str(i))
         f = open(filename, "w")
         if self._app_schema["mode"]=="folder":
            for i in self._pictures_list:
               f.write(i+"\n")
         elif self._app_schema["mode"]=="picture":
            f.write(self._gnome_background_schema["picture-uri"][7:])
         f.close()
      self._pictures_list_editor.run(filename)
