#! /usr/bin/python
# -*- coding=utf-8 -*-

"""
    This file is part of Gnome 3 background slideshow.

    Gnome 3 background slideshow is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Gnome 3 background slideshow is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gnome 3 background slideshow.  If not, see <http://www.gnu.org/licenses/>.
"""

from gi.repository import Gtk, GObject
from informations import *
from DConfWidgets import *
import clioptions

class GeneralConfigTab(Gtk.Table):
   def __init__(self, dialog, applet, gnome_background_schema, app_schema, pixbuf_cache):
      Gtk.Table.__init__(self)
      self.set_border_width(5)
      self.set_col_spacings(10)
      self.set_row_spacings(10)
      
      cb1 = DConfRadioButton(app_schema, "mode", "picture")
      cb1.set_label(_("PICTURE_MODE"))
      self.attach(cb1, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._picture = DConfImageFileChooser(gnome_background_schema, "picture-uri", dialog, pixbuf_cache)
      self.attach(self._picture, 1, 3, 0, 1, yoptions=0)
      
      cb2 = DConfRadioButton(app_schema, "mode", "folder")
      cb2.join_group(cb1)
      cb2.set_label(_("FOLDER_MODE"))
      self.attach(cb2, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._folder = DConfFolderChooser(app_schema, "folder")
      self.attach(self._folder, 1, 2, 1, 2, yoptions=0)
      self._recursive = DConfToggleButton(app_schema, "recursive")
      self._recursive.set_label(_("RECURSIVE"))
      self.attach(self._recursive, 2, 3, 1, 2, yoptions=0)
      
      cb3 = DConfRadioButton(app_schema, "mode", "list")
      cb3.join_group(cb1)
      cb3.set_label(_("LIST_MODE"))
      self.attach(cb3, 0, 1, 2, 3, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._list_filename = DConfFilenameChooser(app_schema, "list-filename")
      self.attach(self._list_filename, 1, 3, 2, 3, yoptions=0)
      
      l = Gtk.Label(_("DELAY"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 3, 4, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._timer = DConfIntegerWidget(app_schema, "delay", 1, 120)
      self.attach(self._timer, 1, 3, 3, 4, yoptions=0)
      
      self._show_tray_icon = DConfToggleButton(app_schema, "show-tray-icon")
      self._show_tray_icon.set_label(_("SHOW_TRAY_ICON"))
      self.attach(self._show_tray_icon, 0, 3, 4, 5, yoptions=0)
      
      if app_schema["mode"]=="picture":
         cb1.set_active(True)
      elif app_schema["mode"]=="folder":
         cb2.set_active(True)
      else:
         cb3.set_active(True)

class OptionConfigTab(Gtk.Table):
   def __init__(self, dialog, applet, gnome_background_schema, app_schema, pixbuf_cache):
      Gtk.Table.__init__(self)
      self.set_border_width(5)
      self.set_col_spacings(10)
      self.set_row_spacings(10)
      
      l = Gtk.Label(_("PICTURE_OPTIONS"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._picture_options = DConfComboxBox(gnome_background_schema, "picture-options")
      self.attach(self._picture_options, 1, 2, 0, 1, yoptions=0)
      
      l = Gtk.Label(_("COLOR_SHADING_TYPE"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 1, 2, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._color_shading_type = DConfComboxBox(gnome_background_schema, "color-shading-type")
      self.attach(self._color_shading_type, 1, 2, 1, 2, yoptions=0)
      
      l = Gtk.Label(_("PRIMARY_COLOR"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 2, 3, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._primary_color = DConfColorChooser(gnome_background_schema, "primary-color")
      self.attach(self._primary_color, 1, 2, 2, 3, yoptions=0)
      
      l = Gtk.Label(_("SECONDARY_COLOR"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 3, 4, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._secondary_color = DConfColorChooser(gnome_background_schema, "secondary-color")
      self.attach(self._secondary_color, 1, 2, 3, 4, yoptions=0)

class AdvancedConfigTab(Gtk.Table):
   def __init__(self, dialog, applet, gnome_background_schema, app_schema, pixbuf_cache):
      Gtk.Table.__init__(self)
      self.set_border_width(5)
      self.set_col_spacings(10)
      self.set_row_spacings(10)
      
      l = Gtk.Label(_("PICTURE_PREVIEW_COMMAND"))
      l.set_alignment(0, 0.5)
      self.attach(l, 0, 1, 0, 1, xoptions=Gtk.AttachOptions.FILL, yoptions=0)
      self._picture_preview_command = DConfEntry(app_schema, "open-pictures-with")
      self.attach(self._picture_preview_command, 1, 2, 0, 1, yoptions=0)

class ConfigDialog(Gtk.Dialog):
   def __init__(self, applet, gnome_background_schema, app_schema, pixbuf_cache):
      Gtk.Dialog.__init__(self)
      self._applet = applet
      self.set_title(_("PREFERENCES"))
      self.add_button(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE)
      self._notebook = Gtk.Notebook()
      self.get_content_area().pack_start(self._notebook, False, False, 0)
      self._general_config_tab = GeneralConfigTab(self, applet, gnome_background_schema, app_schema, pixbuf_cache)
      self._notebook.append_page(self._general_config_tab, Gtk.Label(_("GENERAL_CONFIG")))
      self._options_config_tab = OptionConfigTab(self, applet, gnome_background_schema, app_schema, pixbuf_cache)
      self._notebook.append_page(self._options_config_tab, Gtk.Label(_("OPTIONS_CONFIG")))
      self._advanced_config_tab = AdvancedConfigTab(self, applet, gnome_background_schema, app_schema, pixbuf_cache)
      self._notebook.append_page(self._advanced_config_tab, Gtk.Label(_("ADVANCED_CONFIG")))
      self._progress_bar = Gtk.ProgressBar()
      self.get_content_area().pack_start(self._progress_bar, False, False, 0)
      self._progress_bar.set_no_show_all(True)
   def _check_loading_progress(self):
      progress = self._applet.pictures_list_loading_progress.get()
      if progress!=0 and progress!=1:
         self._progress_bar.show()
         if progress==-1:
            self._progress_bar.pulse()
         else:
            self._progress_bar.set_fraction(progress)
      else:
         self._progress_bar.hide()
      return True
   def rec_list_children(self, root=None):
      if root==None:
         root = self
      res = []
      for i in root.get_children():
         res.append(i)
         if hasattr(i, "get_children"):
            res += self.rec_list_children(i)
      return res
   def start_watching(self):
      for i in self.rec_list_children():
         if issubclass(type(i), DConfWidget):
            i.start_watching()
   def run(self):
      if not clioptions.configure:
         timer = GObject.timeout_add(300, self._check_loading_progress)
      self.start_watching()
      self.show_all()
      Gtk.Dialog.run(self)
      self.hide()
      if not clioptions.configure:
         GObject.source_remove(timer)
