#! /usr/bin/python
# -*- coding=utf-8 -*-

"""
    This file is part of Gnome 3 background slideshow.

    Gnome 3 background slideshow is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Gnome 3 background slideshow is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gnome 3 background slideshow.  If not, see <http://www.gnu.org/licenses/>.
"""

from gi.repository import Gtk, GObject, GdkPixbuf
import thread, os, FileDialogs, sys
from ..ThreadedVariable import ThreadedVariable, ThreadedList

class PicturesListEditor(Gtk.Dialog):
   def __init__(self, applet, app_schema, pixbuf_cache):
      Gtk.Dialog.__init__(self)
      self.set_title(_("PICTURES_LIST_EDITOR"))
      self.set_size_request(500, 350)
      self.maximize()
      self._applet = applet
      self._pixbuf_cache = pixbuf_cache
      self._app_schema = app_schema
      self.add_files_progress = ThreadedVariable(0)
      self.stop_loading = ThreadedVariable(False)
      self.added_files = ThreadedList()
      self.set_border_width(5)
      
      self._add_files_dialog = FileDialogs.AddFilesDialog(self, self._pixbuf_cache)
                  
      self._toolbar = Gtk.Toolbar()
      self.get_content_area().pack_start(self._toolbar, False, True, 0)
      b = Gtk.ToolButton.new_from_stock(Gtk.STOCK_ADD)
      b.connect('clicked', lambda w:self.add_files())
      self._toolbar.insert(b, -1)
      b = Gtk.ToolButton.new_from_stock(Gtk.STOCK_REMOVE)
      b.connect('clicked', lambda w:self.remove_selection())
      self._toolbar.insert(b, -1)
      b = Gtk.ToolButton()
      b.set_icon_name("preview-file")
      b.connect('clicked', lambda w:self.preview_selection())
      self._toolbar.insert(b, -1)
      
      hbox = Gtk.Box()
      hbox.set_border_width(5)
      hbox.set_spacing(10)
      hbox.set_orientation(Gtk.Orientation.HORIZONTAL)
      self.get_content_area().pack_start(hbox, True, True, 0)
      
      scw = Gtk.ScrolledWindow()
      self._icon_view = Gtk.IconView()
      scw.add(self._icon_view)
      scw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
      hbox.pack_start(scw, True, True, 0)
      
      self._preview = Gtk.Image()
      self._preview.set_size_request(300, 300)
      hbox.pack_start(self._preview, False, False, 0)
      
      self._progress_bar = Gtk.ProgressBar()
      self.get_content_area().pack_start(self._progress_bar, False, False, 0)
      self._progress_bar.set_no_show_all(True)
      
      self._buttonbox = Gtk.ButtonBox()
      self._buttonbox.set_layout(Gtk.ButtonBoxStyle.END)
      self._buttonbox.set_spacing(10)
      b = Gtk.Button.new_from_stock(Gtk.STOCK_SAVE)
      self._buttonbox.add(b)
      b.connect("clicked", lambda w:self.response(Gtk.ResponseType.OK))
      b = Gtk.Button.new_from_stock(Gtk.STOCK_CLOSE)
      self._buttonbox.add(b)
      b.connect("clicked", lambda w:self.response(Gtk.ResponseType.CLOSE))
      self.get_content_area().pack_start(self._buttonbox, False, False, 0)
      
      self._list_store = Gtk.ListStore(str, str, GdkPixbuf.Pixbuf)
      self._icon_view.set_model(self._list_store)
      self._icon_view.set_text_column(1)
      self._icon_view.set_pixbuf_column(2)
      
      self._icon_view.set_selection_mode(Gtk.SelectionMode.MULTIPLE)
      
      self._icon_view.connect("selection_changed", lambda w:self._update_preview())
      self._icon_view.connect("key_press_event", self._on_icon_view_key_pressed)
      self._icon_view.connect("item_activated", self._on_icon_item_activated)
   def _on_icon_item_activated(self, widget, path):
      self.preview_picture(self._list_store.get(self._list_store.get_iter(path), 0)[0])
   def _on_icon_view_key_pressed(self, widget, event):
      keyval = event.get_keyval()[1]
      if keyval==65535:
         self.remove_selection()
   def _update_preview(self):
      sel = self._icon_view.get_selected_items()
      self._preview.clear()
      if len(sel)>0:
         filename = self._list_store.get(self._list_store.get_iter(sel[0]), 0)[0]
         p = self._pixbuf_cache.get(filename, 300, 300)
         if p:
            self._preview.set_from_pixbuf(p)
   def _start_loading(self):
      f = open(self._filename)
      files = f.read().splitlines()
      f.close()
      self._process_add_files(files)
   def preview_picture(self, filename):
      command = self._app_schema["open-pictures-with"]
      if command=="":
         command = "xdg-open"
      full_command = None
      for path in os.getenv("PATH").split(":"):
         command_filename = os.path.join(path, command)
         if os.path.exists(command_filename) and os.access(command_filename, os.X_OK):
            full_command = command_filename
            break
      if full_command:
         if os.fork()==0:
            os.execvp(full_command, ("", filename))
      else:
         sys.stderr.write("Command not found: %s\n\r"%command)
   def preview_selection(self):
      sel = self._icon_view.get_selected_items()
      if len(sel)>0:
         filename = self._list_store.get(self._list_store.get_iter(sel[0]), 0)[0]
         self.preview_picture(filename)
   def remove_selection(self):
      for i in self._icon_view.get_selected_items():
         self._list_store.remove(self._list_store.get_iter(i))
   def add_files(self):
      filenames = self._add_files_dialog.run()
      if filenames:
         self._process_add_files(filenames)
   def _process_add_files(self, filenames):
      self._progress_bar.set_fraction(0)
      self.add_files_progress.set(0)
      self._progress_bar.show()
      GObject.timeout_add(100, self._check_files_added)
      thread.start_new_thread(self._add_files, (filenames,))
   def _add_files(self, files):
      n = len(files)
      progress = 0
      for i in files:
         if self.stop_loading.get():
            return
         p = self._pixbuf_cache.get(i, 100, 100)
         if p:
            self.added_files.append((i, os.path.split(i)[1], p))
         progress += 1
         self.add_files_progress.set(1.*progress/n)
      self.add_files_progress.set(1)
   def _check_files_added(self):
      f = self.add_files_progress.get()
      while len(self.added_files):
         self._list_store.append(self.added_files[0])
         del self.added_files[0]
      if f==1:
         self._progress_bar.hide()
         return False
      else:
         self._progress_bar.set_fraction(f)
         return True
   def save(self):
      f = open(self._filename, "w")
      for i in self._list_store:
         f.write(i[0]+"\n")
      f.close()
      self._app_schema.set_string("list-filename", "file://"+self._filename)
      self._app_schema.set_string("mode", "list")
   def run(self, filename):
      self.stop_loading.set(False)
      self._filename = filename
      self._list_store.clear()
      self._preview.clear()
      self._start_loading()
      self.show_all()
      if Gtk.Dialog.run(self)==Gtk.ResponseType.OK:
         self.save()
      self.stop_loading.set(True)
      self.hide()
